@@ -68,7 +68,7 @@ class AnsibleLintRule(BaseRule):
         text = re.sub(r"{#.+?#}", "JINJA_COMMENT", text)
         return text
 
-    # pylint: disable=too-many-arguments
+    # pylint: disable=too-many-arguments,too-many-positional-arguments
     def create_matcherror(
         self,
         message: str = "",
@@ -327,9 +327,9 @@ class TransformMixin:
             # The cast() calls tell mypy what types we expect.
             # Essentially this does:
             if isinstance(segment, str):
-                target = cast(MutableMapping[str, Any], target)[segment]
+                target = cast("MutableMapping[str, Any]", target)[segment]
             elif isinstance(segment, int):
-                target = cast(MutableSequence[Any], target)[segment]
+                target = cast("MutableSequence[Any]", target)[segment]
         return target
 
 
@@ -396,7 +396,10 @@ class RulesCollection:
         else:
             self.options = options
         self.profile = []
-        self.app = app or get_app(cached=True)
+        # app should be defined on normal run logic, but for testing we might
+        # not pass it, and in this case we assume offline mode for performance
+        # reasons.
+        self.app = app or get_app(offline=True)
 
         if profile_name:
             self.profile = PROFILES[profile_name]
@@ -457,7 +460,7 @@ class RulesCollection:
         """Return a rule from inside the collection based on its id."""
         if not isinstance(item, str):
             msg = f"Expected str but got {type(item)} when trying to access rule by it's id"
-            raise RuntimeError(msg)
+            raise TypeError(msg)
         for rule in self.rules:
             if rule.id == item:
                 return rule
@@ -533,8 +536,7 @@ class RulesCollection:
         tags = set()
         for rule in self.rules:
             tags.add(rule.id)
-            for tag in rule.tags:
-                tags.add(tag)
+            tags.update(rule.tags)
         return sorted(tags)
 
     def list_tags(self) -> str:
