@@ -27,22 +27,20 @@ from ansible_compat.runtime import AnsibleWarning
 
 import ansiblelint.skip_utils
 import ansiblelint.utils
-from ansiblelint._internal.rules import (
-    BaseRule,
-    LoadingFailureRule,
-)
-from ansiblelint.app import App, get_app
 from ansiblelint.constants import States
 from ansiblelint.errors import LintWarning, MatchError, WarnSource
-from ansiblelint.file_utils import Lintable, expand_dirs_in_lintables
+from ansiblelint.file_utils import (
+    Lintable,
+    expand_dirs_in_lintables,
+    expand_paths_vars,
+    normpath,
+)
 from ansiblelint.logger import timed_info
 from ansiblelint.rules.syntax_check import OUTPUT_PATTERNS
 from ansiblelint.text import strip_ansi_escape
 from ansiblelint.utils import (
     PLAYBOOK_DIR,
-    _include_children,
-    _roles_children,
-    _taskshandlers_children,
+    HandleChildren,
     parse_examples_from_plugin,
     template,
 )
@@ -50,6 +48,8 @@ from ansiblelint.utils import (
 if TYPE_CHECKING:
     from collections.abc import Callable, Generator
 
+    from ansiblelint._internal.rules import BaseRule
+    from ansiblelint.app import App
     from ansiblelint.config import Options
     from ansiblelint.constants import FileType
     from ansiblelint.rules import RulesCollection
@@ -111,10 +111,12 @@ class Runner:
             checked_files = set()
         self.checked_files = checked_files
 
+        self.app = self.rules.app
+
     def _update_exclude_paths(self, exclude_paths: list[str]) -> None:
         if exclude_paths:
             # These will be (potentially) relative paths
-            paths = ansiblelint.file_utils.expand_paths_vars(exclude_paths)
+            paths = expand_paths_vars(exclude_paths)
             # Since ansiblelint.utils.find_children returns absolute paths,
             # and the list of files we create in `Runner.run` can contain both
             # relative and absolute paths, we need to cover both bases.
@@ -172,23 +174,21 @@ class Runner:
                 if warn.category is DeprecationWarning:
                     continue
                 if warn.category is LintWarning:
-                    filename: None | Lintable = None
                     if isinstance(warn.source, WarnSource):
                         match = MatchError(
                             message=warn.source.message or warn.category.__name__,
                             rule=self.rules["warning"],
-                            filename=warn.source.filename.filename,
+                            lintable=Lintable(warn.source.filename.filename),
                             tag=warn.source.tag,
                             lineno=warn.source.lineno,
                         )
                     else:
-                        filename = warn.source
                         match = MatchError(
                             message=(
                                 warn.message if isinstance(warn.message, str) else "?"
                             ),
                             rule=self.rules["warning"],
-                            filename=str(filename),
+                            lintable=Lintable(str(warn.source)),
                         )
                     matches.append(match)
                     continue
@@ -237,12 +237,12 @@ class Runner:
 
         # -- phase 1 : syntax check in parallel --
         if not self.skip_ansible_syntax_check:
-            app = get_app(offline=True)
+            # app = get_app(cached=True)
 
             def worker(lintable: Lintable) -> list[MatchError]:
                 return self._get_ansible_syntax_check_matches(
                     lintable=lintable,
-                    app=app,
+                    app=self.app,
                 )
 
             for lintable in self.lintables:
@@ -265,28 +265,28 @@ class Runner:
                 matches.extend(data)
 
             matches = self._filter_excluded_matches(matches)
+
         # -- phase 2 ---
-        if not matches:
-            # do our processing only when ansible syntax check passed in order
-            # to avoid causing runtime exceptions. Our processing is not as
-            # resilient to be able process garbage.
-            matches.extend(self._emit_matches(files))
+        # do our processing only when ansible syntax check passed in order
+        # to avoid causing runtime exceptions. Our processing is not as
+        # resilient to be able process garbage.
+        matches.extend(self._emit_matches(files))
 
-            # remove duplicates from files list
-            files = [value for n, value in enumerate(files) if value not in files[:n]]
+        # remove duplicates from files list
+        files = [value for n, value in enumerate(files) if value not in files[:n]]
 
-            for file in self.lintables:
-                if file in self.checked_files or not file.kind:
-                    continue
-                _logger.debug(
-                    "Examining %s of type %s",
-                    ansiblelint.file_utils.normpath(file.path),
-                    file.kind,
-                )
+        for file in self.lintables:
+            if file in self.checked_files or not file.kind or file.failed():
+                continue
+            _logger.debug(
+                "Examining %s of type %s",
+                normpath(file.path),
+                file.kind,
+            )
 
-                matches.extend(
-                    self.rules.run(file, tags=set(self.tags), skip_list=self.skip_list),
-                )
+            matches.extend(
+                self.rules.run(file, tags=set(self.tags), skip_list=self.skip_list),
+            )
 
         # update list of checked files
         self.checked_files.update(self.lintables)
@@ -329,7 +329,12 @@ class Runner:
         name: {lintable.path.expanduser()!s}
 """
                 # pylint: disable=consider-using-with
-                fh = tempfile.NamedTemporaryFile(mode="w", suffix=".yml", prefix="play")
+                fh = tempfile.NamedTemporaryFile(  # noqa: SIM115
+                    mode="w",
+                    suffix=".yml",
+                    prefix="play",
+                    encoding="utf-8",
+                )
                 fh.write(playbook_text)
                 fh.flush()
                 playbook_path = fh.name
@@ -340,8 +345,6 @@ class Runner:
             # [WARNING]: provided hosts list is empty, only localhost is available. Note that the implicit localhost does not match 'all'
             cmd = [
                 "ansible-playbook",
-                "-i",
-                "localhost,",
                 "--syntax-check",
                 playbook_path,
             ]
@@ -357,11 +360,11 @@ class Runner:
             # https://github.com/ansible/ansible-lint/issues/3650
             env["ANSIBLE_INVENTORY_ANY_UNPARSED_IS_FAILED"] = "False"
 
-            run = subprocess.run(
+            run = subprocess.run(  # noqa: S603
                 cmd,
                 stdin=subprocess.PIPE,
                 capture_output=True,
-                shell=False,  # needed when command is a list # noqa: S603
+                shell=False,  # needed when command is a list
                 text=True,
                 check=False,
                 env=env,
@@ -392,13 +395,23 @@ class Runner:
                     details = groups.get("details", "")
                     lineno = int(groups.get("line", 1))
 
-                    if "filename" in groups:
+                    if (
+                        "filename" in groups
+                        and str(lintable.path.absolute()) != groups["filename"]
+                        and lintable.filename != groups["filename"]
+                    ):
+                        # avoids creating a new lintable object if the filename
+                        # is matching as this might prevent Lintable.failed()
+                        # feature from working well.
                         filename = Lintable(groups["filename"])
                     else:
                         filename = lintable
                     column = int(groups.get("column", 1))
 
-                    if pattern.tag == "unknown-module" and app.options.nodeps:
+                    if (
+                        pattern.tag in ("unknown-module", "specific")
+                        and app.options.nodeps
+                    ):
                         ignore_rc = True
                     else:
                         results.append(
@@ -448,6 +461,9 @@ class Runner:
         visited: set[Lintable] = set()
         while visited != self.lintables:
             for lintable in self.lintables - visited:
+                visited.add(lintable)
+                if not lintable.path.exists():
+                    continue
                 try:
                     children = self.find_children(lintable)
                     for child in children:
@@ -458,11 +474,13 @@ class Runner:
                 except MatchError as exc:
                     if not exc.filename:  # pragma: no branch
                         exc.filename = str(lintable.path)
-                    exc.rule = LoadingFailureRule()
+                    exc.rule = self.rules["load-failure"]
                     yield exc
                 except AttributeError:
-                    yield MatchError(lintable=lintable, rule=self.rules["load-failure"])
-                visited.add(lintable)
+                    yield MatchError(
+                        lintable=lintable,
+                        rule=self.rules["load-failure"],
+                    )
 
     def find_children(self, lintable: Lintable) -> list[Lintable]:
         """Traverse children of a single file or folder."""
@@ -481,7 +499,9 @@ class Runner:
             try:
                 playbook_ds = ansiblelint.utils.parse_yaml_from_file(str(lintable.path))
             except AnsibleError as exc:
-                raise SystemExit(exc) from exc
+                msg = f"Loading {lintable.filename} caused an {type(exc).__name__} exception: {exc}, file was ignored."
+                _logger.exception(msg)
+                return []
         results = []
         # playbook_ds can be an AnsibleUnicode string, which we consider invalid
         if isinstance(playbook_ds, str):
@@ -489,7 +509,7 @@ class Runner:
         for item in ansiblelint.utils.playbook_items(playbook_ds):
             # if lintable.kind not in ["playbook"]:
             for child in self.play_children(
-                lintable.path.parent,
+                lintable,
                 item,
                 lintable.kind,
                 playbook_dir,
@@ -510,35 +530,40 @@ class Runner:
                 if path != path_str:
                     child.path = Path(path)
                     child.name = child.path.name
-
                 results.append(child)
         return results
 
     def play_children(
         self,
-        basedir: Path,
+        lintable: Lintable,
         item: tuple[str, Any],
         parent_type: FileType,
         playbook_dir: str,
     ) -> list[Lintable]:
         """Flatten the traversed play tasks."""
         # pylint: disable=unused-argument
-        delegate_map: dict[str, Callable[[str, Any, Any, FileType], list[Lintable]]] = {
-            "tasks": _taskshandlers_children,
-            "pre_tasks": _taskshandlers_children,
-            "post_tasks": _taskshandlers_children,
-            "block": _taskshandlers_children,
-            "include": _include_children,
-            "ansible.builtin.include": _include_children,
-            "import_playbook": _include_children,
-            "ansible.builtin.import_playbook": _include_children,
-            "roles": _roles_children,
-            "dependencies": _roles_children,
-            "handlers": _taskshandlers_children,
-            "include_tasks": _include_children,
-            "ansible.builtin.include_tasks": _include_children,
-            "import_tasks": _include_children,
-            "ansible.builtin.import_tasks": _include_children,
+        basedir = lintable.path.parent
+        handlers = HandleChildren(self.rules, app=self.app)
+
+        delegate_map: dict[
+            str,
+            Callable[[Lintable, Any, Any, FileType], list[Lintable]],
+        ] = {
+            "tasks": handlers.taskshandlers_children,
+            "pre_tasks": handlers.taskshandlers_children,
+            "post_tasks": handlers.taskshandlers_children,
+            "block": handlers.taskshandlers_children,
+            "include": handlers.include_children,
+            "ansible.builtin.include": handlers.include_children,
+            "import_playbook": handlers.import_playbook_children,
+            "ansible.builtin.import_playbook": handlers.import_playbook_children,
+            "roles": handlers.roles_children,
+            "dependencies": handlers.roles_children,
+            "handlers": handlers.taskshandlers_children,
+            "include_tasks": handlers.include_children,
+            "ansible.builtin.include_tasks": handlers.include_children,
+            "import_tasks": handlers.include_children,
+            "ansible.builtin.import_tasks": handlers.include_children,
         }
         (k, v) = item
         add_all_plugin_dirs(str(basedir.resolve()))
@@ -550,7 +575,7 @@ class Runner:
                 {"playbook_dir": PLAYBOOK_DIR or str(basedir.resolve())},
                 fail_on_undefined=False,
             )
-            return delegate_map[k](str(basedir), k, v, parent_type)
+            return delegate_map[k](lintable, k, v, parent_type)
         return []
 
     def plugin_children(self, lintable: Lintable) -> list[Lintable]:
@@ -569,9 +594,10 @@ class Runner:
         examples.line_offset = offset
 
         # pylint: disable=consider-using-with
-        examples.file = NamedTemporaryFile(
+        examples.file = NamedTemporaryFile(  # noqa: SIM115
             mode="w+",
             suffix=f"_{lintable.path.name}.yaml",
+            encoding="utf-8",
         )
         examples.file.write(content)
         examples.file.flush()
